#include <boost/test/included/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/test/parameterized_test.hpp>
#include <boost/bind.hpp>
#include <boost/function.hpp>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>

#include "atmos.h"
#include "profile.h"
#include "weight_factory.h"
#include "weif.h"
#include "utils.h"

using namespace atmos;

using namespace boost::unit_test;

class massdimm_restoration_test {
public:
	typedef profile::functor::weight_type weight_type;
	typedef weight::vector_type vector_type;

	massdimm_restoration_test( const weight_type& weight ): weight_(weight) {
	}

	void operator() ( double altitude, double power ) const {
		using namespace boost::numeric::ublas;

		const vector_type indices = weight_( altitude ) * power;

		vector_type grid(1);
		grid(0) = altitude;

		const profile::functor::mixture_covariance_type covariance = identity_matrix<double>(indices.size()); 

		profile::functor rfun( grid, covariance, 1.0, weight_ );

		const profile::functor::result_type& res = rfun( indices );
		BOOST_CHECK_CLOSE( res.cn2(0), power, 1e-6 );
	}

	weight_type weight_;
};

weight::factory* weight_factory;
massdimm_restoration_test* massdimm_restoration_test_obj;

test_suite* init_unit_test_suite( int argc, char** argv ) {
	test_suite* restoration = BOOST_TEST_SUITE( "restoration_suite" );

	const std::string spectra_dir   = "./spectra";
	const std::string mass_response = "./mass_maid.crv";
	const std::string ccd_response  = "./ccd.crv";
	const std::string spectral_cls  = "a05";

	weight_factory = new weight::factory( spectra_dir, mass_response, ccd_response );

	weight::vector_type inner_diam(atmos::channels);
	weight::vector_type outer_diam(atmos::channels);
	inner_diam(0)=0.0; outer_diam(0)=1.27;
	inner_diam(1)=1.3; outer_diam(1)=2.15;
	inner_diam(2)=2.2; outer_diam(2)=3.85;
	inner_diam(3)=3.9; outer_diam(3)=5.50;
	inner_diam = inner_diam * 14.5*0.1;
	outer_diam = outer_diam * 14.5*0.1;
	double dimm_aperture_base = 0.02;
	double dimm_aperture_size = 0.01;

	massdimm_restoration_test_obj = new massdimm_restoration_test(weight_factory->nullexposure_weight( spectral_cls, inner_diam, outer_diam, dimm_aperture_base, dimm_aperture_size ));

	double powers[] = { 1e-16, 1e-14, 1e-13, 1e-11 };
	double altitudes[] = { 0.0, 0.25, 0.5, 0.75, 1.0, 1.5, 2.0, 3.0, 4.0, 6.0, 8.0, 12.0, 16.0, 25.0 };
	for( int i=0; i < sizeof(powers)/sizeof(powers[0]); ++i ) {
		for (std::size_t j=0; j < sizeof(altitudes)/sizeof(altitudes[1]); ++j) {
			restoration->add(BOOST_TEST_CASE(boost::bind(
				&massdimm_restoration_test::operator(), massdimm_restoration_test_obj, altitudes[j], powers[i]
			)));
		}
	}

	framework::master_test_suite().add( restoration );

	return 0;
}

